# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class ContractUpdateParams(TypedDict):
    include: NotRequired[
        List[
            Literal[
                "billing_settings",
                "one_time_fees",
                "pricing_lines",
                "pricing_overrides",
            ]
        ]
    ]
    """
    Additional fields to include in the response.
    """
    pricing_line_actions: NotRequired[
        List["ContractUpdateParamsPricingLineAction"]
    ]
    """
    Pricing line actions to apply.
    """
    pricing_override_actions: NotRequired[
        List["ContractUpdateParamsPricingOverrideAction"]
    ]
    """
    Pricing override actions to apply.
    """


class ContractUpdateParamsPricingLineAction(TypedDict):
    add: NotRequired["ContractUpdateParamsPricingLineActionAdd"]
    """
    Parameters for adding a pricing line.
    """
    remove: NotRequired["ContractUpdateParamsPricingLineActionRemove"]
    """
    Parameters for removing a pricing line.
    """
    type: Literal["add", "remove", "update"]
    """
    The type of pricing line action.
    """
    update: NotRequired["ContractUpdateParamsPricingLineActionUpdate"]
    """
    Parameters for updating a pricing line.
    """


class ContractUpdateParamsPricingLineActionAdd(TypedDict):
    ends_at: "ContractUpdateParamsPricingLineActionAddEndsAt"
    """
    The end time for the pricing line.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key for the pricing line.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Metadata for the pricing line.
    """
    pricing: "ContractUpdateParamsPricingLineActionAddPricing"
    """
    The pricing configuration for the pricing line.
    """
    starts_at: "ContractUpdateParamsPricingLineActionAddStartsAt"
    """
    The start time for the pricing line.
    """


class ContractUpdateParamsPricingLineActionAddEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingLineActionAddPricing(TypedDict):
    price_details: NotRequired[
        "ContractUpdateParamsPricingLineActionAddPricingPriceDetails"
    ]
    """
    V1 price details. Required if `type` is `price`.
    """
    type: Literal["price"]
    """
    The type of pricing.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetails(TypedDict):
    price: str
    """
    The ID of the V1 price.
    """
    pricing_overrides: NotRequired[
        List[
            "ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverride"
        ]
    ]
    """
    Pricing overrides embedded directly on this pricing line.
    """
    quantity_changes: NotRequired[
        List[
            "ContractUpdateParamsPricingLineActionAddPricingPriceDetailsQuantityChange"
        ]
    ]
    """
    Quantity changes for the pricing line. For now, at most one entry is allowed.
    A quantity change clears all future quantity changes on this pricing line.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverride(
    TypedDict,
):
    ends_at: NotRequired[
        "ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideEndsAt"
    ]
    """
    When the override ends. Defaults to the pricing line's end if not specified.
    """
    lookup_key: NotRequired[str]
    """
    A user-provided lookup key to reference this override.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object.
    """
    overwrite_price: NotRequired[
        "ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideOverwritePrice"
    ]
    """
    Parameters for the overwrite_price override. Required if `type` is `overwrite_price`.
    """
    priority: NotRequired[int]
    """
    The priority of this override relative to others. 0 is highest, 100 is lowest. Defaults to 50.
    """
    starts_at: NotRequired[
        "ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideStartsAt"
    ]
    """
    When the override starts. Defaults to the pricing line's start if not specified.
    """
    type: Literal["overwrite_price"]
    """
    The type of override. Currently only `overwrite_price` is supported.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideEndsAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends. Required if `type` is `timestamp`.
    """
    type: Literal["contract_end", "timestamp"]
    """
    The type of the ends_at.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideOverwritePrice(
    TypedDict,
):
    tiering_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines whether the tiered price should be graduated or volume-based.
    """
    tiers: NotRequired[
        List[
            "ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideOverwritePriceTier"
        ]
    ]
    """
    Each element represents a pricing tier.
    """
    unit_amount: NotRequired[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideOverwritePriceTier(
    TypedDict,
):
    flat_amount: NotRequired[str]
    """
    Price for the entire tier, represented as a decimal string in minor currency units.
    """
    unit_amount: NotRequired[str]
    """
    Per-unit price for units included in this tier, represented as a decimal string in minor currency units.
    """
    up_to_decimal: NotRequired[Decimal]
    """
    Up to and including this quantity will be contained in the tier.
    """
    up_to_inf: NotRequired[Literal["inf"]]
    """
    No upper bound to this tier.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetailsPricingOverrideStartsAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts. Required if `type` is `timestamp`.
    """
    type: Literal["contract_start", "timestamp"]
    """
    The type of the starts_at.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetailsQuantityChange(
    TypedDict,
):
    effective_at: "ContractUpdateParamsPricingLineActionAddPricingPriceDetailsQuantityChangeEffectiveAt"
    """
    When this quantity change takes effect.
    """
    set: Decimal
    """
    The quantity to set.
    """


class ContractUpdateParamsPricingLineActionAddPricingPriceDetailsQuantityChangeEffectiveAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp for the effective at.
    """
    type: Literal["timestamp"]
    """
    The type of the effective at.
    """


class ContractUpdateParamsPricingLineActionAddStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingLineActionRemove(TypedDict):
    id: str
    """
    The ID of the pricing line to remove.
    """


class ContractUpdateParamsPricingLineActionUpdate(TypedDict):
    ends_at: NotRequired["ContractUpdateParamsPricingLineActionUpdateEndsAt"]
    """
    The updated end time for the pricing line.
    """
    id: str
    """
    The ID of the pricing line.
    """
    pricing: NotRequired["ContractUpdateParamsPricingLineActionUpdatePricing"]
    """
    Pricing updates for the pricing line (quantity changes and pricing override actions).
    """
    starts_at: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdateStartsAt"
    ]
    """
    The updated start time for the pricing line.
    """


class ContractUpdateParamsPricingLineActionUpdateEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingLineActionUpdatePricing(TypedDict):
    price_details: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetails"
    ]
    """
    V1 price details. Present when the pricing line type is `price`.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetails(
    TypedDict
):
    pricing_override_actions: NotRequired[
        List[
            "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideAction"
        ]
    ]
    """
    Pricing override actions to apply to the overrides embedded on this pricing line.
    """
    quantity_changes: NotRequired[
        List[
            "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsQuantityChange"
        ]
    ]
    """
    Quantity changes for the pricing line. Setting this clears all future quantity changes.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideAction(
    TypedDict,
):
    add: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAdd"
    ]
    """
    Parameters for adding a pricing line override.
    """
    remove: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionRemove"
    ]
    """
    Parameters for removing a pricing line override.
    """
    type: Literal["add", "remove", "update"]
    """
    The type of pricing line override action.
    """
    update: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionUpdate"
    ]
    """
    Parameters for updating a pricing line override.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAdd(
    TypedDict,
):
    ends_at: "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddEndsAt"
    """
    The end time for the override.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key for the override.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object.
    """
    overwrite_price: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddOverwritePrice"
    ]
    """
    Parameters for an overwrite_price override. Required if `type` is `overwrite_price`.
    """
    priority: NotRequired[int]
    """
    The priority of this override relative to others. 0 is highest, 100 is lowest. Defaults to 50.
    """
    starts_at: "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddStartsAt"
    """
    The start time for the override.
    """
    type: Literal["overwrite_price"]
    """
    The type of override to add.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddEndsAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddOverwritePrice(
    TypedDict,
):
    tiering_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines whether the tiered price should be graduated or volume-based.
    """
    tiers: NotRequired[
        List[
            "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddOverwritePriceTier"
        ]
    ]
    """
    Each element represents a pricing tier.
    """
    unit_amount: NotRequired[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddOverwritePriceTier(
    TypedDict,
):
    flat_amount: NotRequired[str]
    """
    Price for the entire tier, represented as a decimal string in minor currency units.
    """
    unit_amount: NotRequired[str]
    """
    Per-unit price for units included in this tier, represented as a decimal string in minor currency units.
    """
    up_to_decimal: NotRequired[Decimal]
    """
    Up to and including this quantity will be contained in the tier.
    """
    up_to_inf: NotRequired[Literal["inf"]]
    """
    No upper bound to this tier.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionAddStartsAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionRemove(
    TypedDict,
):
    id: NotRequired[str]
    """
    The ID of the pricing line override to remove.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key for the override to remove.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionUpdate(
    TypedDict,
):
    ends_at: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionUpdateEndsAt"
    ]
    """
    The updated end time for the override.
    """
    id: NotRequired[str]
    """
    The ID of the pricing line override to update.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key for the override to update.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of key-value pairs that you can attach to an object.
    """
    starts_at: NotRequired[
        "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionUpdateStartsAt"
    ]
    """
    The updated start time for the override.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionUpdateEndsAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsPricingOverrideActionUpdateStartsAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsQuantityChange(
    TypedDict,
):
    effective_at: "ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsQuantityChangeEffectiveAt"
    """
    When this quantity change takes effect.
    """
    set: Decimal
    """
    The quantity to set.
    """


class ContractUpdateParamsPricingLineActionUpdatePricingPriceDetailsQuantityChangeEffectiveAt(
    TypedDict,
):
    timestamp: NotRequired[str]
    """
    The timestamp for the effective at.
    """
    type: Literal["timestamp"]
    """
    The type of the effective at.
    """


class ContractUpdateParamsPricingLineActionUpdateStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingOverrideAction(TypedDict):
    add: NotRequired["ContractUpdateParamsPricingOverrideActionAdd"]
    """
    Parameters for adding a pricing override.
    """
    remove: NotRequired["ContractUpdateParamsPricingOverrideActionRemove"]
    """
    Parameters for removing a pricing override.
    """
    type: Literal["add", "remove", "update"]
    """
    The type of pricing override action.
    """
    update: NotRequired["ContractUpdateParamsPricingOverrideActionUpdate"]
    """
    Parameters for updating a pricing override.
    """


class ContractUpdateParamsPricingOverrideActionAdd(TypedDict):
    ends_at: "ContractUpdateParamsPricingOverrideActionAddEndsAt"
    """
    The end time for the pricing override.
    """
    lookup_key: NotRequired[str]
    """
    A lookup key for the pricing override.
    """
    multiplier: NotRequired[
        "ContractUpdateParamsPricingOverrideActionAddMultiplier"
    ]
    """
    A multiplier override to add.
    """
    overwrite_price: NotRequired[
        "ContractUpdateParamsPricingOverrideActionAddOverwritePrice"
    ]
    """
    An overwrite price override to add.
    """
    priority: int
    """
    The priority for the pricing override. The highest priority is 0 and the lowest is 100.
    """
    starts_at: "ContractUpdateParamsPricingOverrideActionAddStartsAt"
    """
    The start time for the pricing override.
    """
    type: Literal["multiplier"]
    """
    The type of pricing override to add.
    """


class ContractUpdateParamsPricingOverrideActionAddEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingOverrideActionAddMultiplier(TypedDict):
    criteria: List[
        "ContractUpdateParamsPricingOverrideActionAddMultiplierCriterion"
    ]
    """
    Criteria determining which rates the multiplier applies to.
    """
    factor: str
    """
    The multiplier factor, represented as a decimal string. e.g. "0.8" for a 20% reduction.
    """


class ContractUpdateParamsPricingOverrideActionAddMultiplierCriterion(
    TypedDict,
):
    pricing_line_ids: NotRequired[List[str]]
    """
    Filter by pricing line IDs.
    """
    pricing_line_lookup_keys: NotRequired[List[str]]
    """
    Filter by pricing line lookup keys.
    """
    type: Literal["exclude", "include"]
    """
    Whether to include or exclude items matching these criteria.
    """


class ContractUpdateParamsPricingOverrideActionAddOverwritePrice(TypedDict):
    tiering_mode: NotRequired[Literal["graduated", "volume"]]
    """
    Defines whether the tiered price should be graduated or volume-based.
    """
    tiers: List[
        "ContractUpdateParamsPricingOverrideActionAddOverwritePriceTier"
    ]
    """
    Each element represents a pricing tier.
    """
    unit_amount: NotRequired[str]
    """
    The per-unit amount to be charged, represented as a decimal string in minor currency units.
    """


class ContractUpdateParamsPricingOverrideActionAddOverwritePriceTier(
    TypedDict
):
    flat_amount: NotRequired[str]
    """
    Price for the entire tier, represented as a decimal string in minor currency units.
    """
    unit_amount: NotRequired[str]
    """
    Per-unit price for units included in this tier, represented as a decimal string in minor currency units.
    """
    up_to_decimal: NotRequired[Decimal]
    """
    Up to and including this quantity will be contained in the tier.
    """
    up_to_inf: NotRequired[Literal["inf"]]
    """
    No upper bound to this tier.
    """


class ContractUpdateParamsPricingOverrideActionAddStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """


class ContractUpdateParamsPricingOverrideActionRemove(TypedDict):
    id: str
    """
    The ID of the pricing override to remove.
    """


class ContractUpdateParamsPricingOverrideActionUpdate(TypedDict):
    ends_at: NotRequired[
        "ContractUpdateParamsPricingOverrideActionUpdateEndsAt"
    ]
    """
    The updated end time for the pricing override.
    """
    id: str
    """
    The ID of the pricing override.
    """
    starts_at: NotRequired[
        "ContractUpdateParamsPricingOverrideActionUpdateStartsAt"
    ]
    """
    The updated start time for the pricing override.
    """


class ContractUpdateParamsPricingOverrideActionUpdateEndsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item ends.
    """
    type: Literal["billing_period_end", "timestamp"]
    """
    The type of end time to apply.
    """


class ContractUpdateParamsPricingOverrideActionUpdateStartsAt(TypedDict):
    timestamp: NotRequired[str]
    """
    The timestamp when the item starts.
    """
    type: Literal["billing_period_start", "timestamp"]
    """
    The type of start time to apply.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import NotRequired


class PaymentLocationCapabilityModifyParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    location: str
    """
    The location that the capability enables functionality for.
    """
    requested: bool
    """
    To request a new capability for the location, set this to `true`. You can remove it from the location by passing `false`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.reserve._hold_list_params import (
        HoldListParams as HoldListParams,
    )
    from stripe.params.reserve._hold_retrieve_params import (
        HoldRetrieveParams as HoldRetrieveParams,
    )
    from stripe.params.reserve._plan_retrieve_params import (
        PlanRetrieveParams as PlanRetrieveParams,
    )
    from stripe.params.reserve._release_list_params import (
        ReleaseListParams as ReleaseListParams,
    )
    from stripe.params.reserve._release_retrieve_params import (
        ReleaseRetrieveParams as ReleaseRetrieveParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "HoldListParams": ("stripe.params.reserve._hold_list_params", False),
    "HoldRetrieveParams": (
        "stripe.params.reserve._hold_retrieve_params",
        False,
    ),
    "PlanRetrieveParams": (
        "stripe.params.reserve._plan_retrieve_params",
        False,
    ),
    "ReleaseListParams": ("stripe.params.reserve._release_list_params", False),
    "ReleaseRetrieveParams": (
        "stripe.params.reserve._release_retrieve_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class BusinessProfile(StripeObject):
    """
    The Stripe business profile represents a business' public identity on the Stripe network.
    """

    OBJECT_NAME: ClassVar[Literal["v2.network.business_profile"]] = (
        "v2.network.business_profile"
    )

    class Branding(StripeObject):
        class Icon(StripeObject):
            original: str
            """
            The URL of the image in its original size.
            """

        class Logo(StripeObject):
            original: str
            """
            The URL of the image in its original size.
            """

        icon: Optional[Icon]
        """
        URL of the icon for the business. The image will be square and at least 128px x 128px.
        """
        logo: Optional[Logo]
        """
        URL of the logo for the business. The image will be at least 128px x 128px.
        """
        primary_color: Optional[str]
        """
        A CSS hex color value representing the primary branding color for this business.
        """
        secondary_color: Optional[str]
        """
        A CSS hex color value representing the secondary branding color for this business.
        """
        _inner_class_types = {"icon": Icon, "logo": Logo}

    branding: Optional[Branding]
    """
    Branding data for the business.
    """
    description: Optional[str]
    """
    The description of the business.
    """
    display_name: str
    """
    The display name of the Stripe business profile.
    """
    id: str
    """
    The ID of the Stripe business profile; also known as the Network ID. This is the ID used to identify the business on the Stripe network.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.network.business_profile"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    url: Optional[str]
    """
    The URL of the business.
    """
    username: str
    """
    The username of the Stripe business profile.
    """
    _inner_class_types = {"branding": Branding}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import UntypedStripeObject
from typing import Dict
from typing_extensions import NotRequired, TypedDict


class LicensedItemCreateParams(TypedDict):
    display_name: str
    """
    Description that customers see in the invoice line item.
    Maximum length of 250 characters.
    """
    lookup_key: NotRequired[str]
    """
    An internal key you can use to search for a particular licensed item.
    Must be unique among licensed items.
    Maximum length of 200 characters.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    tax_details: NotRequired["LicensedItemCreateParamsTaxDetails"]
    """
    Stripe Tax details.
    """
    unit_label: NotRequired[str]
    """
    The unit to use when displaying prices for this billable item in places like Checkout. For example, set this field
    to "seat" for Checkout to display "(price) per seat", or "environment" to display "(price) per environment".
    Maximum length of 100 characters.
    """


class LicensedItemCreateParamsTaxDetails(TypedDict):
    tax_code: str
    """
    Product tax code (PTC).
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.billing._one_time_item import OneTimeItem


class RateCardCustomPricingUnitOverageRate(StripeObject):
    """
    A rate card custom pricing unit overage rate defines the conversion rate from a custom pricing unit
    to fiat currency when credits are exhausted. This enables automatic overage charges
    at a configurable per-unit rate.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.billing.rate_card_custom_pricing_unit_overage_rate"]
    ] = "v2.billing.rate_card_custom_pricing_unit_overage_rate"
    created: str
    """
    Timestamp of when the object was created.
    """
    custom_pricing_unit: str
    """
    The ID of the custom pricing unit this overage rate applies to.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.billing.rate_card_custom_pricing_unit_overage_rate"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    one_time_item: "OneTimeItem"
    """
    A one-time item represents a product that can be charged as a one-time line item,
    used for overage charges when custom pricing unit credits are exhausted.
    """
    rate_card: str
    """
    The ID of the Rate Card this overage rate belongs to.
    """
    rate_card_version: str
    """
    The ID of the Rate Card Version this overage rate was created on.
    """
    unit_amount: str
    """
    The per-unit amount to charge for overages, represented as a decimal string in minor currency units with at most 12 decimal places.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class SubscriptionPauseParams(RequestOptions):
    bill_for: NotRequired["SubscriptionPauseParamsBillFor"]
    """
    Controls what to bill for when pausing the subscription.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    invoicing_behavior: NotRequired[Literal["invoice", "pending_invoice_item"]]
    """
    Determines how to handle debits and credits when pausing. Defaults to `pending_invoice_item`.
    """
    type: NotRequired[Literal["subscription"]]
    """
    The type of pause to apply. Defaults to `subscription`.
    """


class SubscriptionPauseParamsBillFor(TypedDict):
    outstanding_usage_through: NotRequired[
        "SubscriptionPauseParamsBillForOutstandingUsageThrough"
    ]
    """
    Controls when to bill for metered usage in the current period. Defaults to `{ type: "now" }`.
    """
    unused_time_from: NotRequired[
        "SubscriptionPauseParamsBillForUnusedTimeFrom"
    ]
    """
    Controls when to credit for unused time on licensed items. Defaults to `{ type: "now" }`.
    """


class SubscriptionPauseParamsBillForOutstandingUsageThrough(TypedDict):
    type: Literal["none", "now"]
    """
    When to bill metered usage in the current period.
    """


class SubscriptionPauseParamsBillForUnusedTimeFrom(TypedDict):
    type: Literal["item_current_period_start", "none", "now"]
    """
    When to credit for unused time.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._api_mode import ApiMode
from stripe._stripe_object import StripeObject
from stripe._stripe_response import StripeResponse
from stripe.v2.core._event import Event, EventNotification
from typing import Any, Dict, List, Optional, cast
from typing_extensions import Literal, TYPE_CHECKING, override

if TYPE_CHECKING:
    from stripe._api_requestor import _APIRequestor


class V1AccountSignalsIncludingDelinquencyCreatedEventNotification(
    EventNotification,
):
    LOOKUP_TYPE = "v1.account_signals[delinquency].created"
    type: Literal["v1.account_signals[delinquency].created"]

    @override
    def fetch_event(
        self,
    ) -> "V1AccountSignalsIncludingDelinquencyCreatedEvent":
        return cast(
            "V1AccountSignalsIncludingDelinquencyCreatedEvent",
            super().fetch_event(),
        )

    @override
    async def fetch_event_async(
        self,
    ) -> "V1AccountSignalsIncludingDelinquencyCreatedEvent":
        return cast(
            "V1AccountSignalsIncludingDelinquencyCreatedEvent",
            await super().fetch_event_async(),
        )


class V1AccountSignalsIncludingDelinquencyCreatedEvent(Event):
    LOOKUP_TYPE = "v1.account_signals[delinquency].created"
    type: Literal["v1.account_signals[delinquency].created"]

    class V1AccountSignalsIncludingDelinquencyCreatedEventData(StripeObject):
        class Indicator(StripeObject):
            description: str
            """
            A brief explanation of how this indicator contributed to the delinquency probability.
            """
            impact: Literal[
                "decrease", "neutral", "slight_increase", "strong_increase"
            ]
            """
            The effect this indicator had on the overall risk level.
            """
            indicator: Literal[
                "account_balance",
                "aov",
                "charge_concentration",
                "disputes",
                "dispute_window",
                "duplicates",
                "exposure",
                "firmographic",
                "lifetime_metrics",
                "payment_processing",
                "payment_volume",
                "payouts",
                "refunds",
                "tenure",
                "transfers",
            ]
            """
            The name of the specific indicator used in the risk assessment.
            """

        account: str
        """
        The account for which the signals belong to.
        """
        evaluated_at: str
        """
        Time at which the signal was evaluated.
        """
        indicators: List[Indicator]
        """
        Array of objects representing individual factors that contributed to the calculated probability of delinquency.
        """
        probability: Optional[Decimal]
        """
        The probability of delinquency. Can be between 0.00 and 100.00.
        """
        risk_level: Literal[
            "elevated", "highest", "low", "normal", "not_assessed", "unknown"
        ]
        """
        Categorical assessment of the delinquency risk based on probability.
        """
        signal_id: str
        """
        Unique identifier for the delinquency signal.
        """
        _inner_class_types = {"indicators": Indicator}
        _field_encodings = {"probability": "decimal_string"}

    data: V1AccountSignalsIncludingDelinquencyCreatedEventData
    """
    Data for the v1.account_signals[delinquency].created event
    """

    @classmethod
    def _construct_from(
        cls,
        *,
        values: Dict[str, Any],
        last_response: Optional[StripeResponse] = None,
        requestor: "_APIRequestor",
        api_mode: ApiMode,
    ) -> "V1AccountSignalsIncludingDelinquencyCreatedEvent":
        evt = super()._construct_from(
            values=values,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )
        if hasattr(evt, "data"):
            evt.data = V1AccountSignalsIncludingDelinquencyCreatedEvent.V1AccountSignalsIncludingDelinquencyCreatedEventData._construct_from(
                values=evt.data,
                last_response=last_response,
                requestor=requestor,
                api_mode=api_mode,
            )
        return evt



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.billing._metered_item_create_params import (
        MeteredItemCreateParams,
    )
    from stripe.params.v2.billing._metered_item_list_params import (
        MeteredItemListParams,
    )
    from stripe.params.v2.billing._metered_item_retrieve_params import (
        MeteredItemRetrieveParams,
    )
    from stripe.params.v2.billing._metered_item_update_params import (
        MeteredItemUpdateParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.billing._metered_item import MeteredItem


class MeteredItemService(StripeService):
    def list(
        self,
        params: Optional["MeteredItemListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[MeteredItem]":
        """
        List all Metered Item objects in reverse chronological order of creation.
        """
        return cast(
            "ListObject[MeteredItem]",
            self._request(
                "get",
                "/v2/billing/metered_items",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["MeteredItemListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[MeteredItem]":
        """
        List all Metered Item objects in reverse chronological order of creation.
        """
        return cast(
            "ListObject[MeteredItem]",
            await self._request_async(
                "get",
                "/v2/billing/metered_items",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "MeteredItemCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "MeteredItem":
        """
        Create a Metered Item object.
        """
        return cast(
            "MeteredItem",
            self._request(
                "post",
                "/v2/billing/metered_items",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "MeteredItemCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "MeteredItem":
        """
        Create a Metered Item object.
        """
        return cast(
            "MeteredItem",
            await self._request_async(
                "post",
                "/v2/billing/metered_items",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["MeteredItemRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "MeteredItem":
        """
        Retrieve a Metered Item object.
        """
        return cast(
            "MeteredItem",
            self._request(
                "get",
                "/v2/billing/metered_items/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["MeteredItemRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "MeteredItem":
        """
        Retrieve a Metered Item object.
        """
        return cast(
            "MeteredItem",
            await self._request_async(
                "get",
                "/v2/billing/metered_items/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["MeteredItemUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "MeteredItem":
        """
        Update a Metered Item object. At least one of the fields is required.
        """
        return cast(
            "MeteredItem",
            self._request(
                "post",
                "/v2/billing/metered_items/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["MeteredItemUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "MeteredItem":
        """
        Update a Metered Item object. At least one of the fields is required.
        """
        return cast(
            "MeteredItem",
            await self._request_async(
                "post",
                "/v2/billing/metered_items/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class OutboundTransferCreateParams(RequestOptions):
    amount: int
    """
    Amount (in cents) to be transferred.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    destination_payment_method: NotRequired[str]
    """
    The PaymentMethod to use as the payment instrument for the OutboundTransfer.
    """
    destination_payment_method_data: NotRequired[
        "OutboundTransferCreateParamsDestinationPaymentMethodData"
    ]
    """
    Hash used to generate the PaymentMethod to be used for this OutboundTransfer. Exclusive with `destination_payment_method`.
    """
    destination_payment_method_options: NotRequired[
        "OutboundTransferCreateParamsDestinationPaymentMethodOptions"
    ]
    """
    Hash describing payment method configuration details.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    financial_account: str
    """
    The FinancialAccount to pull funds from.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    network_details: NotRequired["OutboundTransferCreateParamsNetworkDetails"]
    """
    Details about the network used for the OutboundTransfer.
    """
    statement_descriptor: NotRequired[str]
    """
    Statement descriptor to be shown on the receiving end of an OutboundTransfer. Maximum 10 characters for `ach` transfers or 140 characters for `us_domestic_wire` transfers. The default value is "transfer". Can only include -#.$&*, spaces, and alphanumeric characters.
    """


class OutboundTransferCreateParamsDestinationPaymentMethodData(TypedDict):
    financial_account: NotRequired[str]
    """
    Required if type is set to `financial_account`. The FinancialAccount ID to send funds to.
    """
    type: Literal["financial_account"]
    """
    The type of the destination.
    """


class OutboundTransferCreateParamsDestinationPaymentMethodOptions(TypedDict):
    us_bank_account: NotRequired[
        "Literal['']|OutboundTransferCreateParamsDestinationPaymentMethodOptionsUsBankAccount"
    ]
    """
    Optional fields for `us_bank_account`.
    """


class OutboundTransferCreateParamsDestinationPaymentMethodOptionsUsBankAccount(
    TypedDict,
):
    network: NotRequired[Literal["ach", "us_domestic_wire"]]
    """
    Specifies the network rails to be used. If not set, will default to the PaymentMethod's preferred network. See the [docs](https://docs.stripe.com/treasury/money-movement/timelines) to learn more about money movement timelines for each network type.
    """


class OutboundTransferCreateParamsNetworkDetails(TypedDict):
    ach: NotRequired["OutboundTransferCreateParamsNetworkDetailsAch"]
    """
    Optional fields for `ach`.
    """
    type: Literal["ach"]
    """
    The type of flow that originated the OutboundTransfer.
    """


class OutboundTransferCreateParamsNetworkDetailsAch(TypedDict):
    addenda: NotRequired[str]
    """
    Addenda record data associated with this OutboundTransfer.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class AccountLinkCreateParams(TypedDict):
    account: str
    """
    The ID of the Account to create link for.
    """
    use_case: "AccountLinkCreateParamsUseCase"
    """
    The use case of the AccountLink.
    """


class AccountLinkCreateParamsUseCase(TypedDict):
    account_onboarding: NotRequired[
        "AccountLinkCreateParamsUseCaseAccountOnboarding"
    ]
    """
    Hash containing configuration options for an Account Link object that onboards a new account.
    """
    account_update: NotRequired["AccountLinkCreateParamsUseCaseAccountUpdate"]
    """
    Hash containing configuration options for an Account Link that updates an existing account.
    """
    recipient_onboarding: NotRequired[
        "AccountLinkCreateParamsUseCaseRecipientOnboarding"
    ]
    """
    Hash containing configuration options for an Account Link object that onboards a new recipient.
    """
    recipient_update: NotRequired[
        "AccountLinkCreateParamsUseCaseRecipientUpdate"
    ]
    """
    Hash containing configuration options for an Account Link that updates an existing recipient.
    """
    type: Literal[
        "account_onboarding",
        "account_update",
        "recipient_onboarding",
        "recipient_update",
    ]
    """
    Open Enum. The type of Account Link the user is requesting.
    """


class AccountLinkCreateParamsUseCaseAccountOnboarding(TypedDict):
    collection_options: NotRequired[
        "AccountLinkCreateParamsUseCaseAccountOnboardingCollectionOptions"
    ]
    """
    Specifies the requirements that Stripe collects from v2/core/accounts in the Onboarding flow.
    """
    configurations: List[
        Literal["customer", "merchant", "money_manager", "recipient", "storer"]
    ]
    """
    Open Enum. A v2/core/account can be configured to enable certain functionality. The configuration param targets the v2/core/account_link to collect information for the specified v2/core/account configuration/s.
    """
    refresh_url: str
    """
    The URL the user will be redirected to if the AccountLink is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new AccountLink with the same parameters used to create the original AccountLink, then redirect the user to the new AccountLink's URL so they can continue the flow. If a new AccountLink cannot be generated or the redirect fails you should display a useful error to the user. Please make sure to implement authentication before redirecting the user in case this URL is leaked to a third party.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon completing the linked flow.
    """


class AccountLinkCreateParamsUseCaseAccountOnboardingCollectionOptions(
    TypedDict,
):
    fields: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). If you don't specify collection_options, the default value is currently_due.
    """
    future_requirements: NotRequired[Literal["include", "omit"]]
    """
    Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
    """


class AccountLinkCreateParamsUseCaseAccountUpdate(TypedDict):
    collection_options: NotRequired[
        "AccountLinkCreateParamsUseCaseAccountUpdateCollectionOptions"
    ]
    """
    Specifies the requirements that Stripe collects from v2/core/accounts in the Onboarding flow.
    """
    configurations: List[
        Literal["customer", "merchant", "money_manager", "recipient", "storer"]
    ]
    """
    Open Enum. A v2/account can be configured to enable certain functionality. The configuration param targets the v2/account_link to collect information for the specified v2/account configuration/s.
    """
    refresh_url: str
    """
    The URL the user will be redirected to if the Account Link is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new Account Link with the same parameters used to create the original Account Link, then redirect the user to the new Account Link URL so they can continue the flow. Make sure to authenticate the user before redirecting to the new Account Link, in case the URL leaks to a third party. If a new Account Link can't be generated, or if the redirect fails, you should display a useful error to the user.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon completing the linked flow.
    """


class AccountLinkCreateParamsUseCaseAccountUpdateCollectionOptions(TypedDict):
    fields: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). The default value is `currently_due`.
    """
    future_requirements: NotRequired[Literal["include", "omit"]]
    """
    Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
    """


class AccountLinkCreateParamsUseCaseRecipientOnboarding(TypedDict):
    collection_options: NotRequired[
        "AccountLinkCreateParamsUseCaseRecipientOnboardingCollectionOptions"
    ]
    """
    Specifies the requirements that Stripe collects from v2/core/accounts in the Onboarding flow.
    """
    configurations: List[Literal["recipient"]]
    """
    Open Enum. A v2/core/account can be configured to enable certain functionality. The configuration param targets the v2/core/account_link to collect information for the specified v2/core/account configuration/s.
    """
    refresh_url: str
    """
    The URL the user will be redirected to if the AccountLink is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new AccountLink with the same parameters used to create the original AccountLink, then redirect the user to the new AccountLink's URL so they can continue the flow. If a new AccountLink cannot be generated or the redirect fails you should display a useful error to the user. Please make sure to implement authentication before redirecting the user in case this URL is leaked to a third party.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon completing the linked flow.
    """


class AccountLinkCreateParamsUseCaseRecipientOnboardingCollectionOptions(
    TypedDict,
):
    fields: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). If you don't specify collection_options, the default value is currently_due.
    """
    future_requirements: NotRequired[Literal["include", "omit"]]
    """
    Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
    """


class AccountLinkCreateParamsUseCaseRecipientUpdate(TypedDict):
    collection_options: NotRequired[
        "AccountLinkCreateParamsUseCaseRecipientUpdateCollectionOptions"
    ]
    """
    Specifies the requirements that Stripe collects from v2/core/accounts in the Update flow.
    """
    configurations: List[Literal["recipient"]]
    """
    Open Enum. A v2/account can be configured to enable certain functionality. The configuration param targets the v2/account_link to collect information for the specified v2/account configuration/s.
    """
    refresh_url: str
    """
    The URL the user will be redirected to if the Account Link is expired, has been used, or is otherwise invalid. The URL you specify should attempt to generate a new Account Link with the same parameters used to create the original Account Link, then redirect the user to the new Account Link URL so they can continue the flow. Make sure to authenticate the user before redirecting to the new Account Link, in case the URL leaks to a third party. If a new Account Link can't be generated, or if the redirect fails, you should display a useful error to the user.
    """
    return_url: NotRequired[str]
    """
    The URL that the user will be redirected to upon completing the linked flow.
    """


class AccountLinkCreateParamsUseCaseRecipientUpdateCollectionOptions(
    TypedDict
):
    fields: NotRequired[Literal["currently_due", "eventually_due"]]
    """
    Specifies whether the platform collects only currently_due requirements (`currently_due`) or both currently_due and eventually_due requirements (`eventually_due`). The default value is `currently_due`.
    """
    future_requirements: NotRequired[Literal["include", "omit"]]
    """
    Specifies whether the platform collects future_requirements in addition to requirements in Connect Onboarding. The default value is `omit`.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._balance_transaction import BalanceTransaction
    from stripe.params._tax_fund_list_params import TaxFundListParams
    from stripe.params._tax_fund_retrieve_params import TaxFundRetrieveParams


class TaxFund(ListableAPIResource["TaxFund"]):
    """
    A TaxFund object represents a single tax float sweep event — funds moved between
    a merchant's payments balance and their tax fund financial account for Stripe Tax obligations.
    """

    OBJECT_NAME: ClassVar[Literal["tax_fund"]] = "tax_fund"

    class Context(StripeObject):
        checkout_session: Optional[str]
        """
        The Checkout Session associated with this sweep, if any.
        """
        credit_note: Optional[str]
        """
        The Credit Note associated with this sweep, if any.
        """
        invoice: Optional[str]
        """
        The Invoice associated with this sweep, if any.
        """
        payment_intent: Optional[str]
        """
        The PaymentIntent associated with this sweep, if any.
        """
        refund: Optional[str]
        """
        The Refund associated with this sweep, if any.
        """
        tax_transaction: Optional[str]
        """
        The Tax Transaction associated with this sweep, if any.
        """

    class Destination(StripeObject):
        class PaymentsBalance(StripeObject):
            balance_transaction: ExpandableField["BalanceTransaction"]
            """
            The balance transaction on the payments balance side of the sweep.
            """

        class TaxFundAccount(StripeObject):
            financial_account: Optional[str]
            """
            The tax fund financial account involved in the sweep.
            """
            transaction: Optional[str]
            """
            The financial account transaction on the tax fund account side of the sweep.
            """

        payments_balance: Optional[PaymentsBalance]
        """
        Details about the payments balance side of the sweep.
        """
        tax_fund_account: Optional[TaxFundAccount]
        """
        Details about the tax fund financial account side of the sweep.
        """
        type: str
        """
        The type of account funds moved to or from. Either `payments_balance` or `tax_fund_account`.
        """
        _inner_class_types = {
            "payments_balance": PaymentsBalance,
            "tax_fund_account": TaxFundAccount,
        }

    class Source(StripeObject):
        class PaymentsBalance(StripeObject):
            balance_transaction: ExpandableField["BalanceTransaction"]
            """
            The balance transaction on the payments balance side of the sweep.
            """

        class TaxFundAccount(StripeObject):
            financial_account: Optional[str]
            """
            The tax fund financial account involved in the sweep.
            """
            transaction: Optional[str]
            """
            The financial account transaction on the tax fund account side of the sweep.
            """

        payments_balance: Optional[PaymentsBalance]
        """
        Details about the payments balance side of the sweep.
        """
        tax_fund_account: Optional[TaxFundAccount]
        """
        Details about the tax fund financial account side of the sweep.
        """
        type: str
        """
        The type of account funds moved to or from. Either `payments_balance` or `tax_fund_account`.
        """
        _inner_class_types = {
            "payments_balance": PaymentsBalance,
            "tax_fund_account": TaxFundAccount,
        }

    class Trigger(StripeObject):
        balance_transaction: ExpandableField["BalanceTransaction"]
        """
        The balance transaction on the payments balance that triggered the sweep.
        """
        type: str
        """
        The type of event that triggered the sweep. Always `balance_transaction`.
        """

    amount: int
    """
    Amount swept, in the smallest currency unit. Always positive.
    """
    context: Optional[Context]
    """
    Associated billing or tax documents for this sweep.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    destination: Destination
    """
    Where funds moved to.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["tax_fund"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    source: Source
    """
    Where funds moved from.
    """
    trigger: Trigger
    """
    What caused the sweep.
    """

    @classmethod
    def list(
        cls, **params: Unpack["TaxFundListParams"]
    ) -> ListObject["TaxFund"]:
        """
        Returns a list of tax funds in reverse chronological order.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["TaxFundListParams"]
    ) -> ListObject["TaxFund"]:
        """
        Returns a list of tax funds in reverse chronological order.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["TaxFundRetrieveParams"]
    ) -> "TaxFund":
        """
        Retrieves a tax fund object by its ID.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["TaxFundRetrieveParams"]
    ) -> "TaxFund":
        """
        Retrieves a tax fund object by its ID.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "context": Context,
        "destination": Destination,
        "source": Source,
        "trigger": Trigger,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.payments._off_session_payment_service import (
        OffSessionPaymentService,
    )
    from stripe.v2.payments._settlement_allocation_intent_service import (
        SettlementAllocationIntentService,
    )

_subservices = {
    "off_session_payments": [
        "stripe.v2.payments._off_session_payment_service",
        "OffSessionPaymentService",
    ],
    "settlement_allocation_intents": [
        "stripe.v2.payments._settlement_allocation_intent_service",
        "SettlementAllocationIntentService",
    ],
}


class PaymentsService(StripeService):
    off_session_payments: "OffSessionPaymentService"
    settlement_allocation_intents: "SettlementAllocationIntentService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.billing._alert import Alert


class AlertTriggered(StripeObject):
    OBJECT_NAME: ClassVar[Literal["billing.alert_triggered"]] = (
        "billing.alert_triggered"
    )

    class AggregationPeriod(StripeObject):
        ends_at: int
        """
        End time of the aggregation period
        """
        starts_at: int
        """
        Start time of the aggregation period
        """

    class GroupBy(StripeObject):
        billing_cadence: Optional[str]
        """
        The billing cadence ID, populated when type is `billing_cadence`
        """
        pricing_plan_subscription: Optional[str]
        """
        The pricing plan subscription ID, populated when type is `pricing_plan_subscription`
        """
        type: Literal["billing_cadence", "pricing_plan_subscription"]
        """
        The type of grouping used for this alert notification
        """

    aggregation_period: Optional[AggregationPeriod]
    """
    The aggregation period for which this alert triggered
    """
    alert: "Alert"
    """
    A billing alert is a resource that notifies you when a certain usage threshold on a meter is crossed. For example, you might create a billing alert to notify you when a certain user made 100 API requests.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: Optional[str]
    """
    Currency for the threshold value
    """
    custom_pricing_unit: Optional[str]
    """
    Custom pricing unit for the threshold value
    """
    customer: str
    """
    ID of customer for which the alert triggered
    """
    external_customer_id: Optional[str]
    """
    External customer ID for the customer for which the alert triggered
    """
    group_by: Optional[GroupBy]
    """
    Populated specifically for spend alerts to notify merchants which group_by entity has the exceeded spend
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["billing.alert_triggered"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    triggered_at: Optional[int]
    """
    Timestamp at which the threshold was crossed
    """
    value: Decimal
    """
    The value triggering the alert
    """
    _inner_class_types = {
        "aggregation_period": AggregationPeriod,
        "group_by": GroupBy,
    }
    _field_encodings = {"value": "decimal_string"}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class FinancialAccountListParams(TypedDict):
    include: NotRequired[List[Literal["payments.balance_by_funds_type"]]]
    """
    Additional fields to include in the response.
    """
    limit: NotRequired[int]
    """
    The page limit.
    """
    statuses: NotRequired[List[Literal["closed", "open", "pending"]]]
    """
    Filter for FinancialAccount `status`. By default, closed FinancialAccounts are not returned.
    """
    types: NotRequired[
        List[
            Literal[
                "accrued_fees",
                "credit",
                "multiprocessor_settlement",
                "payments",
                "storage",
            ]
        ]
    ]
    """
    Filter for FinancialAccount `type`. By default, FinancialAccounts of any `type` are returned.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.money_management._adjustment_service import (
        AdjustmentService,
    )
    from stripe.v2.money_management._currency_conversion_service import (
        CurrencyConversionService,
    )
    from stripe.v2.money_management._debit_dispute_service import (
        DebitDisputeService,
    )
    from stripe.v2.money_management._financial_account_service import (
        FinancialAccountService,
    )
    from stripe.v2.money_management._financial_address_service import (
        FinancialAddressService,
    )
    from stripe.v2.money_management._inbound_transfer_service import (
        InboundTransferService,
    )
    from stripe.v2.money_management._outbound_payment_quote_service import (
        OutboundPaymentQuoteService,
    )
    from stripe.v2.money_management._outbound_payment_service import (
        OutboundPaymentService,
    )
    from stripe.v2.money_management._outbound_setup_intent_service import (
        OutboundSetupIntentService,
    )
    from stripe.v2.money_management._outbound_transfer_service import (
        OutboundTransferService,
    )
    from stripe.v2.money_management._payout_intent_service import (
        PayoutIntentService,
    )
    from stripe.v2.money_management._payout_method_service import (
        PayoutMethodService,
    )
    from stripe.v2.money_management._payout_methods_bank_account_spec_service import (
        PayoutMethodsBankAccountSpecService,
    )
    from stripe.v2.money_management._received_credit_service import (
        ReceivedCreditService,
    )
    from stripe.v2.money_management._received_debit_service import (
        ReceivedDebitService,
    )
    from stripe.v2.money_management._recipient_verification_service import (
        RecipientVerificationService,
    )
    from stripe.v2.money_management._test_helpers_service import (
        TestHelpersService,
    )
    from stripe.v2.money_management._transaction_entry_service import (
        TransactionEntryService,
    )
    from stripe.v2.money_management._transaction_service import (
        TransactionService,
    )

_subservices = {
    "adjustments": [
        "stripe.v2.money_management._adjustment_service",
        "AdjustmentService",
    ],
    "currency_conversions": [
        "stripe.v2.money_management._currency_conversion_service",
        "CurrencyConversionService",
    ],
    "debit_disputes": [
        "stripe.v2.money_management._debit_dispute_service",
        "DebitDisputeService",
    ],
    "financial_accounts": [
        "stripe.v2.money_management._financial_account_service",
        "FinancialAccountService",
    ],
    "financial_addresses": [
        "stripe.v2.money_management._financial_address_service",
        "FinancialAddressService",
    ],
    "inbound_transfers": [
        "stripe.v2.money_management._inbound_transfer_service",
        "InboundTransferService",
    ],
    "outbound_payments": [
        "stripe.v2.money_management._outbound_payment_service",
        "OutboundPaymentService",
    ],
    "outbound_payment_quotes": [
        "stripe.v2.money_management._outbound_payment_quote_service",
        "OutboundPaymentQuoteService",
    ],
    "outbound_setup_intents": [
        "stripe.v2.money_management._outbound_setup_intent_service",
        "OutboundSetupIntentService",
    ],
    "outbound_transfers": [
        "stripe.v2.money_management._outbound_transfer_service",
        "OutboundTransferService",
    ],
    "payout_intents": [
        "stripe.v2.money_management._payout_intent_service",
        "PayoutIntentService",
    ],
    "payout_methods": [
        "stripe.v2.money_management._payout_method_service",
        "PayoutMethodService",
    ],
    "payout_methods_bank_account_spec": [
        "stripe.v2.money_management._payout_methods_bank_account_spec_service",
        "PayoutMethodsBankAccountSpecService",
    ],
    "received_credits": [
        "stripe.v2.money_management._received_credit_service",
        "ReceivedCreditService",
    ],
    "received_debits": [
        "stripe.v2.money_management._received_debit_service",
        "ReceivedDebitService",
    ],
    "recipient_verifications": [
        "stripe.v2.money_management._recipient_verification_service",
        "RecipientVerificationService",
    ],
    "test_helpers": [
        "stripe.v2.money_management._test_helpers_service",
        "TestHelpersService",
    ],
    "transactions": [
        "stripe.v2.money_management._transaction_service",
        "TransactionService",
    ],
    "transaction_entries": [
        "stripe.v2.money_management._transaction_entry_service",
        "TransactionEntryService",
    ],
}


class MoneyManagementService(StripeService):
    adjustments: "AdjustmentService"
    currency_conversions: "CurrencyConversionService"
    debit_disputes: "DebitDisputeService"
    financial_accounts: "FinancialAccountService"
    financial_addresses: "FinancialAddressService"
    inbound_transfers: "InboundTransferService"
    outbound_payments: "OutboundPaymentService"
    outbound_payment_quotes: "OutboundPaymentQuoteService"
    outbound_setup_intents: "OutboundSetupIntentService"
    outbound_transfers: "OutboundTransferService"
    payout_intents: "PayoutIntentService"
    payout_methods: "PayoutMethodService"
    payout_methods_bank_account_spec: "PayoutMethodsBankAccountSpecService"
    received_credits: "ReceivedCreditService"
    received_debits: "ReceivedDebitService"
    recipient_verifications: "RecipientVerificationService"
    test_helpers: "TestHelpersService"
    transactions: "TransactionService"
    transaction_entries: "TransactionEntryService"

    def __init__(self, requestor):
        super().__init__(requestor)

    def __getattr__(self, name):
        try:
            import_from, service = _subservices[name]
            service_class = getattr(
                import_module(import_from),
                service,
            )
            setattr(
                self,
                name,
                service_class(self._requestor),
            )
            return getattr(self, name)
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject, UntypedStripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class RateCard(StripeObject):
    """
    A Rate Card represents a versioned set of usage-based prices (rates). Each rate is associated with one Metered Item and
    defines how much to charge for usage of that item. After you've set up a RateCard, you can subscribe customers to it
    by creating a Rate Card Subscription.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.rate_card"]] = (
        "v2.billing.rate_card"
    )

    class ServiceCycle(StripeObject):
        interval: Literal["day", "month", "week", "year"]
        """
        The interval for assessing service.
        """
        interval_count: int
        """
        The length of the interval for assessing service. For example, set this to 3 and `interval` to `"month"`
        to specify quarterly service.
        """

    active: bool
    """
    Whether this RateCard is active. Inactive RateCards cannot be used in new activations or have new rates added.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    currency: str
    """
    Three-letter ISO currency code, in lowercase. Must be a supported currency.
    """
    display_name: str
    """
    A customer-facing name for the Rate Card.
    This name is used in Stripe-hosted products like the Customer Portal and Checkout. It does not show up on Invoices.
    Maximum length of 250 characters.
    """
    id: str
    """
    Unique identifier for the object.
    """
    live_version: str
    """
    The ID of the Rate Card Version used by all subscriptions when no specific version is specified.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    lookup_key: Optional[str]
    """
    An internal key you can use to search for a particular RateCard. Maximum length of 200 characters.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["v2.billing.rate_card"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    service_cycle: ServiceCycle
    """
    The service cycle configuration for this Rate Card. For example, a monthly Rate Card with a rate of 1 USD for the
    first 10 "workloads" and 2 USD thereafter means "1 USD per workload up to 10 workloads during a month of service."
    This is similar to but distinct from billing interval; the service interval deals with the rate at which the
    customer accumulates fees, while the billing interval in Cadence deals with the rate the customer is billed.
    """
    tax_behavior: Literal["exclusive", "inclusive"]
    """
    The tax behavior for Stripe Tax — whether the rate card price includes or excludes tax.
    """
    _inner_class_types = {"service_cycle": ServiceCycle}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.v2.data import analytics as analytics, reporting as reporting
    from stripe.v2.data._analytics_service import (
        AnalyticsService as AnalyticsService,
    )
    from stripe.v2.data._reporting_service import (
        ReportingService as ReportingService,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "analytics": ("stripe.v2.data.analytics", True),
    "reporting": ("stripe.v2.data.reporting", True),
    "AnalyticsService": ("stripe.v2.data._analytics_service", False),
    "ReportingService": ("stripe.v2.data._reporting_service", False),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import UntypedStripeObject
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import sanitize_id
from typing import ClassVar, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.issuing._program_create_params import (
        ProgramCreateParams,
    )
    from stripe.params.issuing._program_list_params import ProgramListParams
    from stripe.params.issuing._program_modify_params import (
        ProgramModifyParams,
    )
    from stripe.params.issuing._program_retrieve_params import (
        ProgramRetrieveParams,
    )


class Program(
    CreateableAPIResource["Program"],
    ListableAPIResource["Program"],
    UpdateableAPIResource["Program"],
):
    """
    An Issuing `Program` represents a card program that the user has access to.
    """

    OBJECT_NAME: ClassVar[Literal["issuing.program"]] = "issuing.program"
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    id: str
    """
    Unique identifier for the object.
    """
    is_default: bool
    """
    Whether or not this is the "default" issuing program new cards are created on. Only one active `is_default` program at the same time.
    """
    metadata: UntypedStripeObject[str]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["issuing.program"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    platform_program: Optional[str]
    """
    The platform's Issuing Program for which this program is associated.
    """

    @classmethod
    def create(cls, **params: Unpack["ProgramCreateParams"]) -> "Program":
        """
        Create a Program object.
        """
        return cast(
            "Program",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["ProgramCreateParams"]
    ) -> "Program":
        """
        Create a Program object.
        """
        return cast(
            "Program",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["ProgramListParams"]
    ) -> ListObject["Program"]:
        """
        List all of the programs the given Issuing user has access to.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["ProgramListParams"]
    ) -> ListObject["Program"]:
        """
        List all of the programs the given Issuing user has access to.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["ProgramModifyParams"]
    ) -> "Program":
        """
        Updates a Program object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Program",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["ProgramModifyParams"]
    ) -> "Program":
        """
        Updates a Program object.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Program",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["ProgramRetrieveParams"]
    ) -> "Program":
        """
        Retrieves the program specified by the given id.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["ProgramRetrieveParams"]
    ) -> "Program":
        """
        Retrieves the program specified by the given id.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import Literal, NotRequired, TypedDict


class OutboundSetupIntentUpdateParams(TypedDict):
    payout_method: NotRequired[str]
    """
    If provided, the existing payout method resource to link to this outbound setup intent.
    """
    payout_method_data: NotRequired[
        "OutboundSetupIntentUpdateParamsPayoutMethodData"
    ]
    """
    If no payout_method provided, used to create the underlying credential that is set up for outbound money movement.
    If a payout_method provided, used to update data on the credential linked to this setup intent.
    Only card expiry (exp_month, exp_year) can be updated in the case where payout_method is provided.
    """


class OutboundSetupIntentUpdateParamsPayoutMethodData(TypedDict):
    bank_account: NotRequired[
        "OutboundSetupIntentUpdateParamsPayoutMethodDataBankAccount"
    ]
    """
    The type specific details of the bank account payout method.
    """
    card: NotRequired["OutboundSetupIntentUpdateParamsPayoutMethodDataCard"]
    """
    The type specific details of the card payout method.
    """
    type: Literal["bank_account", "card", "crypto_wallet"]
    """
    Closed Enum. The type of payout method to be created/updated.
    """


class OutboundSetupIntentUpdateParamsPayoutMethodDataBankAccount(TypedDict):
    account_number: str
    """
    The account number or IBAN of the bank account.
    """
    bank_account_type: NotRequired[
        Literal["checking", "futsu", "savings", "toza"]
    ]
    """
    Closed Enum. The type of the bank account (checking or savings).
    """
    branch_number: NotRequired[str]
    """
    The branch number of the bank account, if present.
    """
    country: str
    """
    The country code of the bank account.
    """
    currency: str
    """
    The currency of the bank account.
    """
    routing_number: NotRequired[str]
    """
    The routing number of the bank account, if present.
    """
    swift_code: NotRequired[str]
    """
    The swift code of the bank account, if present.
    """


class OutboundSetupIntentUpdateParamsPayoutMethodDataCard(TypedDict):
    currency: NotRequired[str]
    """
    The currency of the card.
    """
    exp_month: NotRequired[str]
    """
    The expiration month of the card.
    """
    exp_year: NotRequired[str]
    """
    The expiration year of the card.
    """
    number: NotRequired[str]
    """
    The card number. This can only be passed when creating a new credential on an outbound setup intent in the requires_payout_method state.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.extend._workflow_invoke_params import (
        WorkflowInvokeParams,
    )
    from stripe.params.v2.extend._workflow_list_params import (
        WorkflowListParams,
    )
    from stripe.params.v2.extend._workflow_retrieve_params import (
        WorkflowRetrieveParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.extend._workflow import Workflow
    from stripe.v2.extend._workflow_run import WorkflowRun


class WorkflowService(StripeService):
    def list(
        self,
        params: Optional["WorkflowListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Workflow]":
        """
        List all Workflows.
        """
        return cast(
            "ListObject[Workflow]",
            self._request(
                "get",
                "/v2/extend/workflows",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["WorkflowListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Workflow]":
        """
        List all Workflows.
        """
        return cast(
            "ListObject[Workflow]",
            await self._request_async(
                "get",
                "/v2/extend/workflows",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["WorkflowRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Workflow":
        """
        Retrieves the details of a Workflow by ID.
        """
        return cast(
            "Workflow",
            self._request(
                "get",
                "/v2/extend/workflows/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["WorkflowRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Workflow":
        """
        Retrieves the details of a Workflow by ID.
        """
        return cast(
            "Workflow",
            await self._request_async(
                "get",
                "/v2/extend/workflows/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def invoke(
        self,
        id: str,
        params: "WorkflowInvokeParams",
        options: Optional["RequestOptions"] = None,
    ) -> "WorkflowRun":
        """
        Invokes an on-demand Workflow with parameters, to launch a new Workflow Run.
        """
        return cast(
            "WorkflowRun",
            self._request(
                "post",
                "/v2/extend/workflows/{id}/invoke".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def invoke_async(
        self,
        id: str,
        params: "WorkflowInvokeParams",
        options: Optional["RequestOptions"] = None,
    ) -> "WorkflowRun":
        """
        Invokes an on-demand Workflow with parameters, to launch a new Workflow Run.
        """
        return cast(
            "WorkflowRun",
            await self._request_async(
                "post",
                "/v2/extend/workflows/{id}/invoke".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_resource import APIResource
from stripe._list_object import ListObject
from stripe._nested_resource_class_methods import nested_resource_class_methods
from stripe._stripe_object import StripeObject
from stripe._util import sanitize_id
from typing import ClassVar, List, Optional, cast
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.crypto._customer_consumer_wallet import CustomerConsumerWallet
    from stripe.crypto._customer_payment_token import CustomerPaymentToken
    from stripe.params.crypto._customer_list_crypto_consumer_wallets_params import (
        CustomerListCryptoConsumerWalletsParams,
    )
    from stripe.params.crypto._customer_list_payment_tokens_params import (
        CustomerListPaymentTokensParams,
    )
    from stripe.params.crypto._customer_retrieve_params import (
        CustomerRetrieveParams,
    )


@nested_resource_class_methods("crypto_consumer_wallet")
@nested_resource_class_methods("payment_token")
class Customer(APIResource["Customer"]):
    """
    This object represents a crypto onramp customer. Use it to get their kyc status and payment methods.
    """

    OBJECT_NAME: ClassVar[Literal["crypto.customer"]] = "crypto.customer"

    class KycTier(StripeObject):
        tier: Literal["l0", "l1", "l2"]
        """
        The KYC tier level (e.g., l0, l1, l2).
        """
        verification_errors: List[
            Literal[
                "id_document_verification_failed",
                "phone_verification_failed",
                "user_has_reached_max_verification_attempts",
            ]
        ]
        """
        List of errors associated with this KYC tier verification.
        """
        verification_status: Literal[
            "not_available", "not_started", "pending", "rejected", "verified"
        ]
        """
        The verification status for this KYC tier.
        """

    class Verification(StripeObject):
        errors: List[
            Literal[
                "id_document_verification_failed",
                "phone_verification_failed",
                "user_has_reached_max_verification_attempts",
            ]
        ]
        """
        List of errors associated with the verification.
        """
        name: Literal["id_document_verified", "kyc_verified", "phone_verified"]
        """
        Type of verification.
        """
        status: Literal[
            "not_available", "not_started", "pending", "rejected", "verified"
        ]
        """
        Outcome of the verification.
        """

    id: str
    """
    Unique identifier for the object.
    """
    kyc_region: Optional[Literal["eu", "us"]]
    """
    The KYC region determined by the customer's address country.
    """
    kyc_tiers: List[KycTier]
    """
    List of KYC tiers and their verification status.
    """
    object: Literal["crypto.customer"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    provided_fields: List[
        Literal[
            "address_city",
            "address_country",
            "address_line_1",
            "address_line_2",
            "address_postal_code",
            "address_state",
            "attestation",
            "birth_city",
            "birth_country",
            "dob",
            "first_name",
            "id_document",
            "id_number",
            "id_type",
            "identifiers",
            "last_name",
            "nationalities",
            "selfie",
        ]
    ]
    """
    The set of KYC Fields provided for this customers.
    """
    verifications: List[Verification]
    """
    List of verifications and their outcome.
    """

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["CustomerRetrieveParams"]
    ) -> "Customer":
        """
        Retrieves the details of a Crypto Customer.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["CustomerRetrieveParams"]
    ) -> "Customer":
        """
        Retrieves the details of a Crypto Customer.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def list_crypto_consumer_wallets(
        cls,
        id: str,
        **params: Unpack["CustomerListCryptoConsumerWalletsParams"],
    ) -> ListObject["CustomerConsumerWallet"]:
        """
        Lists the Consumer Wallets for a Crypto Customer.
        """
        return cast(
            ListObject["CustomerConsumerWallet"],
            cls._static_request(
                "get",
                "/v1/crypto/customers/{id}/crypto_consumer_wallets".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    async def list_crypto_consumer_wallets_async(
        cls,
        id: str,
        **params: Unpack["CustomerListCryptoConsumerWalletsParams"],
    ) -> ListObject["CustomerConsumerWallet"]:
        """
        Lists the Consumer Wallets for a Crypto Customer.
        """
        return cast(
            ListObject["CustomerConsumerWallet"],
            await cls._static_request_async(
                "get",
                "/v1/crypto/customers/{id}/crypto_consumer_wallets".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    def list_payment_tokens(
        cls, id: str, **params: Unpack["CustomerListPaymentTokensParams"]
    ) -> ListObject["CustomerPaymentToken"]:
        """
        Lists the Payment Tokens for a Crypto Customer.
        """
        return cast(
            ListObject["CustomerPaymentToken"],
            cls._static_request(
                "get",
                "/v1/crypto/customers/{id}/payment_tokens".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @classmethod
    async def list_payment_tokens_async(
        cls, id: str, **params: Unpack["CustomerListPaymentTokensParams"]
    ) -> ListObject["CustomerPaymentToken"]:
        """
        Lists the Payment Tokens for a Crypto Customer.
        """
        return cast(
            ListObject["CustomerPaymentToken"],
            await cls._static_request_async(
                "get",
                "/v1/crypto/customers/{id}/payment_tokens".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    _inner_class_types = {"kyc_tiers": KycTier, "verifications": Verification}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import NotRequired, TypedDict


class TransactionListParams(TypedDict):
    created: NotRequired[str]
    """
    Filter for Transactions created at an exact time.
    """
    created_gt: NotRequired[str]
    """
    Filter for Transactions created after the specified timestamp.
    """
    created_gte: NotRequired[str]
    """
    Filter for Transactions created at or after the specified timestamp.
    """
    created_lt: NotRequired[str]
    """
    Filter for Transactions created before the specified timestamp.
    """
    created_lte: NotRequired[str]
    """
    Filter for Transactions created at or before the specified timestamp.
    """
    financial_account: NotRequired[str]
    """
    Filter for Transactions belonging to a FinancialAccount.
    """
    flow: NotRequired[str]
    """
    Filter for Transactions corresponding to a Flow.
    """
    limit: NotRequired[int]
    """
    The page limit.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._bank_account import BankAccount
    from stripe._card import Card
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._external_account_create_params import (
        ExternalAccountCreateParams,
    )
    from stripe.params._external_account_delete_params import (
        ExternalAccountDeleteParams,
    )
    from stripe.params._external_account_list_params import (
        ExternalAccountListParams,
    )
    from stripe.params._external_account_retrieve_params import (
        ExternalAccountRetrieveParams,
    )
    from stripe.params._external_account_update_params import (
        ExternalAccountUpdateParams,
    )
    from typing import Union


class ExternalAccountService(StripeService):
    def delete(
        self,
        id: str,
        params: Optional["ExternalAccountDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Delete a specified external account for a given account.
        """
        return cast(
            "Union[BankAccount, Card]",
            self._request(
                "delete",
                "/v1/external_accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        id: str,
        params: Optional["ExternalAccountDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Delete a specified external account for a given account.
        """
        return cast(
            "Union[BankAccount, Card]",
            await self._request_async(
                "delete",
                "/v1/external_accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["ExternalAccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Retrieve a specified external account for a given account.
        """
        return cast(
            "Union[BankAccount, Card]",
            self._request(
                "get",
                "/v1/external_accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["ExternalAccountRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Retrieve a specified external account for a given account.
        """
        return cast(
            "Union[BankAccount, Card]",
            await self._request_async(
                "get",
                "/v1/external_accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["ExternalAccountUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Updates the metadata, account holder name, account holder type of a bank account belonging to
        a connected account and optionally sets it as the default for its currency. Other bank account
        details are not editable by design.

        You can only update bank accounts when [account.controller.requirement_collection is application, which includes <a href="/connect/custom-accounts">Custom accounts](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection).

        You can re-enable a disabled bank account by performing an update call without providing any
        arguments or changes.
        """
        return cast(
            "Union[BankAccount, Card]",
            self._request(
                "post",
                "/v1/external_accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["ExternalAccountUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Updates the metadata, account holder name, account holder type of a bank account belonging to
        a connected account and optionally sets it as the default for its currency. Other bank account
        details are not editable by design.

        You can only update bank accounts when [account.controller.requirement_collection is application, which includes <a href="/connect/custom-accounts">Custom accounts](https://docs.stripe.com/api/accounts/object#account_object-controller-requirement_collection).

        You can re-enable a disabled bank account by performing an update call without providing any
        arguments or changes.
        """
        return cast(
            "Union[BankAccount, Card]",
            await self._request_async(
                "post",
                "/v1/external_accounts/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        params: Optional["ExternalAccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Union[BankAccount, Card]]":
        """
        List external accounts for an account.
        """
        return cast(
            "ListObject[Union[BankAccount, Card]]",
            self._request(
                "get",
                "/v1/external_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ExternalAccountListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Union[BankAccount, Card]]":
        """
        List external accounts for an account.
        """
        return cast(
            "ListObject[Union[BankAccount, Card]]",
            await self._request_async(
                "get",
                "/v1/external_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "ExternalAccountCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Create an external account for a given connected account.
        """
        return cast(
            "Union[BankAccount, Card]",
            self._request(
                "post",
                "/v1/external_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ExternalAccountCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Union[BankAccount, Card]":
        """
        Create an external account for a given connected account.
        """
        return cast(
            "Union[BankAccount, Card]",
            await self._request_async(
                "post",
                "/v1/external_accounts",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.params.v2.money_management._outbound_setup_intent_cancel_params import (
        OutboundSetupIntentCancelParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_create_params import (
        OutboundSetupIntentCreateParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_list_params import (
        OutboundSetupIntentListParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_retrieve_params import (
        OutboundSetupIntentRetrieveParams,
    )
    from stripe.params.v2.money_management._outbound_setup_intent_update_params import (
        OutboundSetupIntentUpdateParams,
    )
    from stripe.v2._list_object import ListObject
    from stripe.v2.money_management._outbound_setup_intent import (
        OutboundSetupIntent,
    )


class OutboundSetupIntentService(StripeService):
    def list(
        self,
        params: Optional["OutboundSetupIntentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OutboundSetupIntent]":
        """
        List the OutboundSetupIntent objects.
        """
        return cast(
            "ListObject[OutboundSetupIntent]",
            self._request(
                "get",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["OutboundSetupIntentListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[OutboundSetupIntent]":
        """
        List the OutboundSetupIntent objects.
        """
        return cast(
            "ListObject[OutboundSetupIntent]",
            await self._request_async(
                "get",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["OutboundSetupIntentCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Create an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            self._request(
                "post",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["OutboundSetupIntentCreateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Create an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            await self._request_async(
                "post",
                "/v2/money_management/outbound_setup_intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["OutboundSetupIntentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Retrieve an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            self._request(
                "get",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["OutboundSetupIntentRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Retrieve an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            await self._request_async(
                "get",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["OutboundSetupIntentUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Update an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            self._request(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["OutboundSetupIntentUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Update an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            await self._request_async(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        id: str,
        params: Optional["OutboundSetupIntentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Cancel an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            self._request(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        id: str,
        params: Optional["OutboundSetupIntentCancelParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "OutboundSetupIntent":
        """
        Cancel an OutboundSetupIntent object.
        """
        return cast(
            "OutboundSetupIntent",
            await self._request_async(
                "post",
                "/v2/money_management/outbound_setup_intents/{id}/cancel".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._list_object import ListObject
from stripe._stripe_object import StripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.tax._calculation_create_params import (
        CalculationCreateParams,
    )
    from stripe.params.tax._calculation_list_line_items_params import (
        CalculationListLineItemsParams,
    )
    from stripe.params.tax._calculation_retrieve_params import (
        CalculationRetrieveParams,
    )
    from stripe.tax._calculation_line_item import CalculationLineItem


class Calculation(CreateableAPIResource["Calculation"]):
    """
    A Tax Calculation allows you to calculate the tax to collect from your customer.

    Related guide: [Calculate tax in your custom payment flow](https://docs.stripe.com/tax/custom)
    """

    OBJECT_NAME: ClassVar[Literal["tax.calculation"]] = "tax.calculation"

    class CustomerDetails(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix, such as "NY" or "TX".
            """

        class TaxId(StripeObject):
            type: Literal[
                "ad_nrt",
                "ae_trn",
                "al_tin",
                "am_tin",
                "ao_tin",
                "ar_cuit",
                "au_abn",
                "au_arn",
                "aw_tin",
                "az_tin",
                "ba_tin",
                "bb_tin",
                "bd_bin",
                "bf_ifu",
                "bg_uic",
                "bh_vat",
                "bj_ifu",
                "bo_tin",
                "br_cnpj",
                "br_cpf",
                "bs_tin",
                "by_tin",
                "ca_bn",
                "ca_gst_hst",
                "ca_pst_bc",
                "ca_pst_mb",
                "ca_pst_sk",
                "ca_qst",
                "cd_nif",
                "ch_uid",
                "ch_vat",
                "cl_tin",
                "cm_niu",
                "cn_tin",
                "co_nit",
                "cr_tin",
                "cv_nif",
                "de_stn",
                "do_rcn",
                "ec_ruc",
                "eg_tin",
                "es_cif",
                "et_tin",
                "eu_oss_vat",
                "eu_vat",
                "fo_vat",
                "gb_vat",
                "ge_vat",
                "gi_tin",
                "gn_nif",
                "hk_br",
                "hr_oib",
                "hu_tin",
                "id_npwp",
                "il_vat",
                "in_gst",
                "is_vat",
                "it_cf",
                "jp_cn",
                "jp_rn",
                "jp_trn",
                "ke_pin",
                "kg_tin",
                "kh_tin",
                "kr_brn",
                "kz_bin",
                "la_tin",
                "li_uid",
                "li_vat",
                "lk_vat",
                "ma_vat",
                "md_vat",
                "me_pib",
                "mk_vat",
                "mr_nif",
                "mx_rfc",
                "my_frp",
                "my_itn",
                "my_sst",
                "ng_tin",
                "no_vat",
                "no_voec",
                "np_pan",
                "nz_gst",
                "om_vat",
                "pe_ruc",
                "ph_tin",
                "pl_nip",
                "py_ruc",
                "ro_tin",
                "rs_pib",
                "ru_inn",
                "ru_kpp",
                "sa_vat",
                "sg_gst",
                "sg_uen",
                "si_tin",
                "sn_ninea",
                "sr_fin",
                "sv_nit",
                "th_vat",
                "tj_tin",
                "tr_tin",
                "tw_vat",
                "tz_vat",
                "ua_vat",
                "ug_tin",
                "unknown",
                "us_ein",
                "uy_ruc",
                "uz_tin",
                "uz_vat",
                "ve_rif",
                "vn_tin",
                "za_vat",
                "zm_tin",
                "zw_tin",
            ]
            """
            The type of the tax ID, one of `ad_nrt`, `ar_cuit`, `eu_vat`, `bo_tin`, `br_cnpj`, `br_cpf`, `cn_tin`, `co_nit`, `cr_tin`, `do_rcn`, `ec_ruc`, `eu_oss_vat`, `hr_oib`, `pe_ruc`, `ro_tin`, `rs_pib`, `sv_nit`, `uy_ruc`, `ve_rif`, `vn_tin`, `gb_vat`, `nz_gst`, `au_abn`, `au_arn`, `in_gst`, `no_vat`, `no_voec`, `za_vat`, `ch_vat`, `mx_rfc`, `sg_uen`, `ru_inn`, `ru_kpp`, `ca_bn`, `hk_br`, `es_cif`, `pl_nip`, `it_cf`, `fo_vat`, `gi_tin`, `py_ruc`, `tw_vat`, `th_vat`, `jp_cn`, `jp_rn`, `jp_trn`, `li_uid`, `li_vat`, `lk_vat`, `my_itn`, `us_ein`, `kr_brn`, `ca_qst`, `ca_gst_hst`, `ca_pst_bc`, `ca_pst_mb`, `ca_pst_sk`, `my_sst`, `sg_gst`, `ae_trn`, `cl_tin`, `sa_vat`, `id_npwp`, `my_frp`, `il_vat`, `ge_vat`, `ua_vat`, `is_vat`, `bg_uic`, `hu_tin`, `si_tin`, `ke_pin`, `tr_tin`, `eg_tin`, `ph_tin`, `al_tin`, `bh_vat`, `kz_bin`, `ng_tin`, `om_vat`, `de_stn`, `ch_uid`, `tz_vat`, `uz_vat`, `uz_tin`, `md_vat`, `ma_vat`, `by_tin`, `ao_tin`, `bs_tin`, `bb_tin`, `cd_nif`, `mr_nif`, `me_pib`, `zw_tin`, `ba_tin`, `gn_nif`, `mk_vat`, `sr_fin`, `sn_ninea`, `am_tin`, `np_pan`, `tj_tin`, `ug_tin`, `zm_tin`, `kh_tin`, `aw_tin`, `az_tin`, `bd_bin`, `bj_ifu`, `et_tin`, `kg_tin`, `la_tin`, `cm_niu`, `cv_nif`, `bf_ifu`, or `unknown`
            """
            value: str
            """
            The value of the tax ID.
            """

        address: Optional[Address]
        """
        The customer's postal address (for example, home or business location).
        """
        address_source: Optional[Literal["billing", "shipping"]]
        """
        The type of customer address provided.
        """
        ip_address: Optional[str]
        """
        The customer's IP address (IPv4 or IPv6).
        """
        tax_ids: List[TaxId]
        """
        The customer's tax IDs (for example, EU VAT numbers).
        """
        taxability_override: Literal[
            "customer_exempt", "none", "reverse_charge"
        ]
        """
        The taxability override used for taxation.
        """
        _inner_class_types = {"address": Address, "tax_ids": TaxId}

    class ShipFromDetails(StripeObject):
        class Address(StripeObject):
            city: Optional[str]
            """
            City, district, suburb, town, or village.
            """
            country: str
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            line1: Optional[str]
            """
            Address line 1, such as the street, PO Box, or company name.
            """
            line2: Optional[str]
            """
            Address line 2, such as the apartment, suite, unit, or building.
            """
            postal_code: Optional[str]
            """
            ZIP or postal code.
            """
            state: Optional[str]
            """
            State/province as an [ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2) subdivision code, without country prefix, such as "NY" or "TX".
            """

        address: Address
        _inner_class_types = {"address": Address}

    class ShippingCost(StripeObject):
        class TaxBreakdown(StripeObject):
            class Jurisdiction(StripeObject):
                country: str
                """
                Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                """
                display_name: str
                """
                A human-readable name for the jurisdiction imposing the tax.
                """
                level: Literal[
                    "city", "country", "county", "district", "state"
                ]
                """
                Indicates the level of the jurisdiction imposing the tax.
                """
                state: Optional[str]
                """
                [ISO 3166-2 subdivision code](https://en.wikipedia.org/wiki/ISO_3166-2), without country prefix. For example, "NY" for New York, United States.
                """

            class TaxRateDetails(StripeObject):
                display_name: str
                """
                A localized display name for tax type, intended to be human-readable. For example, "Local Sales and Use Tax", "Value-added tax (VAT)", or "Umsatzsteuer (USt.)".
                """
                percentage_decimal: str
                """
                The tax rate percentage as a string. For example, 8.5% is represented as "8.5".
                """
                tax_type: Literal[
                    "admissions_tax",
                    "amusement_tax",
                    "attendance_tax",
                    "communications_tax",
                    "entertainment_tax",
                    "gross_receipts_tax",
                    "gst",
                    "hospitality_tax",
                    "hst",
                    "igst",
                    "jct",
                    "lease_tax",
                    "luxury_tax",
                    "pst",
                    "qst",
                    "resort_tax",
                    "retail_delivery_fee",
                    "rst",
                    "sales_tax",
                    "service_tax",
                    "tourism_tax",
                    "vat",
                ]
                """
                The tax type, such as `vat` or `sales_tax`.
                """

            amount: int
            """
            The amount of tax, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
            """
            jurisdiction: Jurisdiction
            sourcing: Literal["destination", "origin", "performance"]
            """
            Indicates whether the jurisdiction was determined by the origin (merchant's address) or destination (customer's address).
            """
            tax_rate_details: Optional[TaxRateDetails]
            """
            Details regarding the rate for this tax. This field will be `null` when the tax is not imposed, for example if the product is exempt from tax.
            """
            taxability_reason: Literal[
                "customer_exempt",
                "not_collecting",
                "not_subject_to_tax",
                "not_supported",
                "portion_product_exempt",
                "portion_reduced_rated",
                "portion_standard_rated",
                "product_exempt",
                "product_exempt_holiday",
                "proportionally_rated",
                "reduced_rated",
                "reverse_charge",
                "standard_rated",
                "taxable_basis_reduced",
                "zero_rated",
            ]
            """
            The reasoning behind this tax, for example, if the product is tax exempt. The possible values for this field may be extended as new tax rules are supported.
            """
            taxable_amount: int
            """
            The amount on which tax is calculated, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
            """
            _inner_class_types = {
                "jurisdiction": Jurisdiction,
                "tax_rate_details": TaxRateDetails,
            }

        amount: int
        """
        The shipping amount in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units). If `tax_behavior=inclusive`, then this amount includes taxes. Otherwise, taxes were calculated on top of this amount.
        """
        amount_tax: int
        """
        The amount of tax calculated for shipping, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
        """
        shipping_rate: Optional[str]
        """
        The ID of an existing [ShippingRate](https://docs.stripe.com/api/shipping_rates/object).
        """
        tax_behavior: Literal["exclusive", "inclusive"]
        """
        Specifies whether the `amount` includes taxes. If `tax_behavior=inclusive`, then the amount includes taxes.
        """
        tax_breakdown: Optional[List[TaxBreakdown]]
        """
        Detailed account of taxes relevant to shipping cost.
        """
        tax_code: str
        """
        The [tax code](https://docs.stripe.com/tax/tax-categories) ID used for shipping.
        """
        _inner_class_types = {"tax_breakdown": TaxBreakdown}

    class TaxBreakdown(StripeObject):
        class TaxRateDetails(StripeObject):
            class FlatAmount(StripeObject):
                amount: int
                """
                Amount of the tax when the `rate_type` is `flat_amount`. This positive integer represents how much to charge in the smallest currency unit (e.g., 100 cents to charge $1.00 or 100 to charge ¥100, a zero-decimal currency). The amount value supports up to eight digits (e.g., a value of 99999999 for a USD charge of $999,999.99).
                """
                currency: str
                """
                Three-letter ISO currency code, in lowercase.
                """

            country: Optional[str]
            """
            Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
            """
            flat_amount: Optional[FlatAmount]
            """
            The amount of the tax rate when the `rate_type` is `flat_amount`. Tax rates with `rate_type` `percentage` can vary based on the transaction, resulting in this field being `null`. This field exposes the amount and currency of the flat tax rate.
            """
            percentage_decimal: str
            """
            The tax rate percentage as a string. For example, 8.5% is represented as `"8.5"`.
            """
            rate_type: Optional[Literal["flat_amount", "percentage"]]
            """
            Indicates the type of tax rate applied to the taxable amount. This value can be `null` when no tax applies to the location. This field is only present for TaxRates created by Stripe Tax.
            """
            state: Optional[str]
            """
            State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
            """
            tax_type: Optional[
                Literal[
                    "admissions_tax",
                    "amusement_tax",
                    "attendance_tax",
                    "communications_tax",
                    "entertainment_tax",
                    "gross_receipts_tax",
                    "gst",
                    "hospitality_tax",
                    "hst",
                    "igst",
                    "jct",
                    "lease_tax",
                    "luxury_tax",
                    "pst",
                    "qst",
                    "resort_tax",
                    "retail_delivery_fee",
                    "rst",
                    "sales_tax",
                    "service_tax",
                    "tourism_tax",
                    "vat",
                ]
            ]
            """
            The tax type, such as `vat` or `sales_tax`.
            """
            _inner_class_types = {"flat_amount": FlatAmount}

        amount: int
        """
        The amount of tax, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
        """
        inclusive: bool
        """
        Specifies whether the tax amount is included in the line item amount.
        """
        tax_rate_details: TaxRateDetails
        taxability_reason: Literal[
            "customer_exempt",
            "not_collecting",
            "not_subject_to_tax",
            "not_supported",
            "portion_product_exempt",
            "portion_reduced_rated",
            "portion_standard_rated",
            "product_exempt",
            "product_exempt_holiday",
            "proportionally_rated",
            "reduced_rated",
            "reverse_charge",
            "standard_rated",
            "taxable_basis_reduced",
            "zero_rated",
        ]
        """
        The reasoning behind this tax, for example, if the product is tax exempt. We might extend the possible values for this field to support new tax rules.
        """
        taxable_amount: int
        """
        The amount on which tax is calculated, in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
        """
        _inner_class_types = {"tax_rate_details": TaxRateDetails}

    amount_total: int
    """
    Total amount after taxes in the [smallest currency unit](https://docs.stripe.com/currencies#minor-units).
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    customer: Optional[str]
    """
    The ID of an existing [Customer](https://docs.stripe.com/api/customers/object) used for the resource.
    """
    customer_details: CustomerDetails
    expires_at: Optional[int]
    """
    Timestamp of date at which the tax calculation will expire.
    """
    id: Optional[str]
    """
    Unique identifier for the calculation.
    """
    line_items: Optional[ListObject["CalculationLineItem"]]
    """
    The list of items the customer is purchasing.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["tax.calculation"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    ship_from_details: Optional[ShipFromDetails]
    """
    The details of the ship from location, such as the address.
    """
    shipping_cost: Optional[ShippingCost]
    """
    The shipping cost details for the calculation.
    """
    tax_amount_exclusive: int
    """
    The amount of tax to be collected on top of the line item prices.
    """
    tax_amount_inclusive: int
    """
    The amount of tax already included in the line item prices.
    """
    tax_breakdown: List[TaxBreakdown]
    """
    Breakdown of individual tax amounts that add up to the total.
    """
    tax_date: int
    """
    The calculation uses the tax rules and rates that are in effect at this timestamp. You can use a date up to 31 days in the past or up to 31 days in the future. If you use a future date, Stripe doesn't guarantee that the expected tax rules and rate being used match the actual rules and rate that will be in effect on that date. We deploy tax changes before their effective date, but not within a fixed window.
    """

    @classmethod
    def create(
        cls, **params: Unpack["CalculationCreateParams"]
    ) -> "Calculation":
        """
        Calculates tax based on the input and returns a Tax Calculation object.
        """
        return cast(
            "Calculation",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["CalculationCreateParams"]
    ) -> "Calculation":
        """
        Calculates tax based on the input and returns a Tax Calculation object.
        """
        return cast(
            "Calculation",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_list_line_items(
        cls,
        calculation: str,
        **params: Unpack["CalculationListLineItemsParams"],
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            cls._static_request(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(calculation)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def list_line_items(
        calculation: str, **params: Unpack["CalculationListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @overload
    def list_line_items(
        self, **params: Unpack["CalculationListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @class_method_variant("_cls_list_line_items")
    def list_line_items(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CalculationListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            self._request(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_list_line_items_async(
        cls,
        calculation: str,
        **params: Unpack["CalculationListLineItemsParams"],
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            await cls._static_request_async(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(calculation)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def list_line_items_async(
        calculation: str, **params: Unpack["CalculationListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @overload
    async def list_line_items_async(
        self, **params: Unpack["CalculationListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        ...

    @class_method_variant("_cls_list_line_items_async")
    async def list_line_items_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["CalculationListLineItemsParams"]
    ) -> ListObject["CalculationLineItem"]:
        """
        Retrieves the line items of a tax calculation as a collection, if the calculation hasn't expired.
        """
        return cast(
            ListObject["CalculationLineItem"],
            await self._request_async(
                "get",
                "/v1/tax/calculations/{calculation}/line_items".format(
                    calculation=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["CalculationRetrieveParams"]
    ) -> "Calculation":
        """
        Retrieves a Tax Calculation object, if the calculation hasn't expired.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["CalculationRetrieveParams"]
    ) -> "Calculation":
        """
        Retrieves a Tax Calculation object, if the calculation hasn't expired.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {
        "customer_details": CustomerDetails,
        "ship_from_details": ShipFromDetails,
        "shipping_cost": ShippingCost,
        "tax_breakdown": TaxBreakdown,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class CadenceSpendModifier(StripeObject):
    """
    A Spend Modifier changes how spend is computed when billing for specific cadence.
    """

    OBJECT_NAME: ClassVar[Literal["v2.billing.cadence_spend_modifier"]] = (
        "v2.billing.cadence_spend_modifier"
    )

    class MaxBillingPeriodSpend(StripeObject):
        class Amount(StripeObject):
            class CustomPricingUnit(StripeObject):
                id: Optional[str]
                """
                The id of the custom pricing unit.
                """
                value: str
                """
                The decimal value of custom pricing units, represented as a string.
                """

            custom_pricing_unit: Optional[CustomPricingUnit]
            """
            The custom pricing unit amount. Present if type is `custom_pricing_unit`.
            """
            type: Literal["custom_pricing_unit"]
            """
            The type of the spend modifier amount.
            """
            _inner_class_types = {"custom_pricing_unit": CustomPricingUnit}

        class CustomPricingUnitOverageRate(StripeObject):
            id: str
            """
            ID of the custom pricing unit overage rate.
            """

        alert: str
        """
        The billing alert associated with the maximum spend threshold.
        """
        amount: Amount
        """
        The maximum amount of invoice spend.
        """
        custom_pricing_unit_overage_rate: CustomPricingUnitOverageRate
        """
        The configuration for the overage rate for the custom pricing unit.
        """
        effective_from: str
        """
        The timestamp from which this spend modifier is effective.
        """
        effective_until: Optional[str]
        """
        The timestamp until which this spend modifier is effective. If not set, the modifier is effective indefinitely.
        """
        _inner_class_types = {
            "amount": Amount,
            "custom_pricing_unit_overage_rate": CustomPricingUnitOverageRate,
        }

    billing_cadence: str
    """
    The ID of the Billing Cadence this spend modifier is associated with.
    """
    created: str
    """
    Timestamp of when the object was created.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    max_billing_period_spend: Optional[MaxBillingPeriodSpend]
    """
    Max invoice spend details. Present if type is `max_billing_period_spend`.
    """
    object: Literal["v2.billing.cadence_spend_modifier"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    type: Literal["max_billing_period_spend"]
    """
    The type of the spend modifier.
    """
    _inner_class_types = {"max_billing_period_spend": MaxBillingPeriodSpend}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_object import UntypedStripeObject
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class RequestedSessionCreateParams(RequestOptions):
    affiliate_attribution: NotRequired[
        "RequestedSessionCreateParamsAffiliateAttribution"
    ]
    """
    Affiliate attribution data associated with this requested session.
    """
    currency: str
    """
    The currency for this requested session.
    """
    customer: NotRequired[str]
    """
    The customer for this requested session.
    """
    discounts: NotRequired["RequestedSessionCreateParamsDiscounts"]
    """
    The discount codes to apply to this requested session.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    fulfillment_details: NotRequired[
        "RequestedSessionCreateParamsFulfillmentDetails"
    ]
    """
    The details of the fulfillment.
    """
    line_item_details: List["RequestedSessionCreateParamsLineItemDetail"]
    """
    The details of the line items.
    """
    metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    The metadata for this requested session.
    """
    payment_method: NotRequired[str]
    """
    The payment method for this requested session.
    """
    payment_method_options: NotRequired[
        "RequestedSessionCreateParamsPaymentMethodOptions"
    ]
    """
    The payment method options for this requested session.
    """
    seller_details: "RequestedSessionCreateParamsSellerDetails"
    """
    The details of the seller.
    """
    setup_future_usage: NotRequired[Literal["on_session"]]
    """
    The setup future usage for this requested session.
    """
    shared_metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    The shared metadata for this requested session.
    """


class RequestedSessionCreateParamsAffiliateAttribution(TypedDict):
    campaign_id: NotRequired[str]
    """
    Agent-scoped campaign identifier.
    """
    creative_id: NotRequired[str]
    """
    Agent-scoped creative identifier.
    """
    expires_at: int
    """
    Timestamp when the attribution token expires.
    """
    identification_token: str
    """
    Agent-issued secret to validate the legitimacy of the source of this data.
    """
    issued_at: int
    """
    Timestamp for when the attribution token was issued.
    """
    provider: str
    """
    Identifier for the attribution agent / affiliate network namespace.
    """
    publisher_id: NotRequired[str]
    """
    Agent-scoped affiliate/publisher identifier.
    """
    shared_metadata: NotRequired["Dict[str, str]|UntypedStripeObject[str]"]
    """
    Freeform key/value pairs for additional non-sensitive per-agent data.
    """
    source: NotRequired[
        "RequestedSessionCreateParamsAffiliateAttributionSource"
    ]
    """
    Context about where the attribution originated.
    """
    sub_id: NotRequired[str]
    """
    Agent-scoped sub-tracking identifier.
    """
    touchpoint: Literal["first", "last"]
    """
    Whether this is the first or last touchpoint.
    """


class RequestedSessionCreateParamsAffiliateAttributionSource(TypedDict):
    platform: NotRequired[str]
    """
    The platform where the attribution originated.
    """
    type: Literal["platform", "url"]
    """
    The type of the attribution source.
    """
    url: NotRequired[str]
    """
    The URL where the attribution originated.
    """


class RequestedSessionCreateParamsDiscounts(TypedDict):
    codes: List[str]
    """
    Array of discount codes to apply.
    """
    enforce_strict_eligibility: NotRequired[bool]
    """
    Whether to enforce strict eligibility for discount codes. Defaults to true. When false, invalid codes are returned in the discounts.invalid array instead of raising an error.
    """


class RequestedSessionCreateParamsFulfillmentDetails(TypedDict):
    address: NotRequired[
        "RequestedSessionCreateParamsFulfillmentDetailsAddress"
    ]
    """
    The customer's address.
    """
    email: NotRequired[str]
    """
    The customer's email address.
    """
    name: NotRequired[str]
    """
    The customer's name.
    """
    phone: NotRequired[str]
    """
    The customer's phone number.
    """


class RequestedSessionCreateParamsFulfillmentDetailsAddress(TypedDict):
    city: str
    """
    City, district, suburb, town, or village.
    """
    country: str
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: str
    """
    ZIP or postal code.
    """
    state: str
    """
    State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
    """


class RequestedSessionCreateParamsLineItemDetail(TypedDict):
    quantity: int
    """
    The quantity of the line item.
    """
    sku_id: str
    """
    The SKU ID of the line item.
    """


class RequestedSessionCreateParamsPaymentMethodOptions(TypedDict):
    card: NotRequired["RequestedSessionCreateParamsPaymentMethodOptionsCard"]
    """
    Card-specific payment method options.
    """
    excluded_payment_method_types: NotRequired[
        List[Literal["affirm", "card", "klarna"]]
    ]
    """
    The payment method types to exclude from the session.
    """


class RequestedSessionCreateParamsPaymentMethodOptionsCard(TypedDict):
    brands_blocked: NotRequired[
        List[Literal["american_express", "mastercard", "visa"]]
    ]
    """
    The card brands to exclude from the session.
    """


class RequestedSessionCreateParamsSellerDetails(TypedDict):
    network_profile: str
    """
    The network profile for the seller.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from stripe.v2._amount import Amount
from typing import ClassVar, Optional
from typing_extensions import Literal


class DebitDispute(StripeObject):
    """
    A DebitDispute represents a dispute raised against a received debit.
    """

    OBJECT_NAME: ClassVar[Literal["v2.money_management.debit_dispute"]] = (
        "v2.money_management.debit_dispute"
    )

    class BankTransfer(StripeObject):
        network: Literal["ach"]
        """
        The bank network the dispute was originated on.
        """
        reason: Optional[Literal["incorrect_amount_or_date", "unauthorized"]]
        """
        The reason for the dispute.
        """
        statement_descriptor: Optional[str]
        """
        The statement descriptor set by the originator of the debit.
        """

    class StatusDetails(StripeObject):
        class Failed(StripeObject):
            reason: Literal["unknown"]
            """
            The reason for the failure of the DebitDispute.
            """

        failed: Failed
        """
        Information that elaborates on the `failed` status of a DebitDispute.
        It is only present when the DebitDispute status is `failed`.
        """
        _inner_class_types = {"failed": Failed}

    class StatusTransitions(StripeObject):
        failed_at: Optional[str]
        """
        The time when the DebitDispute was marked as `failed`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: `2026-04-23T13:22:18.123Z`.
        """
        succeeded_at: Optional[str]
        """
        The time when the DebitDispute was marked as `succeeded`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: `2026-04-23T13:22:18.123Z`.
        """

    amount: Amount
    """
    The amount of the DebitDispute.
    """
    bank_transfer: Optional[BankTransfer]
    """
    Details about the bank transfer dispute. Present if `type` field value is `bank_transfer`.
    """
    created: str
    """
    Time at which the DebitDispute was created.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: `2026-03-23T13:22:18.123Z`.
    """
    financial_account: str
    """
    The Financial Account associated with the DebitDispute.
    """
    id: str
    """
    The ID of a DebitDispute.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["v2.money_management.debit_dispute"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    received_debit: str
    """
    The ID of the ReceivedDebit that was disputed.
    """
    status: Literal["failed", "submitted", "succeeded"]
    """
    The status of the DebitDispute.
    """
    status_details: Optional[StatusDetails]
    """
    Detailed information about the status of the DebitDispute.
    """
    status_transitions: Optional[StatusTransitions]
    """
    The time at which the DebitDispute transitioned to a particular status.
    """
    type: Literal["bank_transfer"]
    """
    The type of the DebitDispute.
    """
    _inner_class_types = {
        "bank_transfer": BankTransfer,
        "status_details": StatusDetails,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import Literal, NotRequired, TypedDict


class StatementRetrieveParams(TypedDict):
    include: NotRequired[List[Literal["files_by_currency"]]]
    """
    Additional fields to include in the response.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from decimal import Decimal
from stripe._api_mode import ApiMode
from stripe._stripe_object import StripeObject
from stripe._stripe_response import StripeResponse
from stripe.v2.core._event import Event, EventNotification
from typing import Any, Dict, List, Optional, cast
from typing_extensions import Literal, TYPE_CHECKING, override

if TYPE_CHECKING:
    from stripe._api_requestor import _APIRequestor


class V2SignalsAccountSignalFraudulentMerchantReadyEventNotification(
    EventNotification,
):
    LOOKUP_TYPE = "v2.signals.account_signal.fraudulent_merchant_ready"
    type: Literal["v2.signals.account_signal.fraudulent_merchant_ready"]

    @override
    def fetch_event(
        self,
    ) -> "V2SignalsAccountSignalFraudulentMerchantReadyEvent":
        return cast(
            "V2SignalsAccountSignalFraudulentMerchantReadyEvent",
            super().fetch_event(),
        )

    @override
    async def fetch_event_async(
        self,
    ) -> "V2SignalsAccountSignalFraudulentMerchantReadyEvent":
        return cast(
            "V2SignalsAccountSignalFraudulentMerchantReadyEvent",
            await super().fetch_event_async(),
        )


class V2SignalsAccountSignalFraudulentMerchantReadyEvent(Event):
    LOOKUP_TYPE = "v2.signals.account_signal.fraudulent_merchant_ready"
    type: Literal["v2.signals.account_signal.fraudulent_merchant_ready"]

    class V2SignalsAccountSignalFraudulentMerchantReadyEventData(StripeObject):
        class FraudulentMerchant(StripeObject):
            class Indicator(StripeObject):
                description: str
                """
                A brief explanation of how this indicator contributed to the fraudulent merchant probability.
                """
                impact: Literal[
                    "decrease", "neutral", "slight_increase", "strong_increase"
                ]
                """
                The effect this indicator had on the overall risk level.
                """
                indicator: Literal[
                    "bank_account",
                    "business_information_and_account_activity",
                    "disputes",
                    "failures",
                    "geolocation",
                    "other",
                    "other_related_accounts",
                    "other_transaction_activity",
                    "owner_email",
                    "web_presence",
                ]
                """
                The name of the specific indicator used in the risk assessment.
                """

            indicators: List[Indicator]
            """
            Array of objects representing individual factors that contributed to the calculated probability. Maximum of 3.
            """
            probability: Optional[Decimal]
            """
            The probability of the merchant being fraudulent. Can be between 0.00 and 100.00. May be empty if the risk_level is UNKNOWN or NOT_ASSESSED.
            """
            risk_level: Literal[
                "elevated",
                "highest",
                "low",
                "normal",
                "not_assessed",
                "unknown",
            ]
            """
            Categorical assessment of the fraudulent merchant risk based on probability.
            """
            _inner_class_types = {"indicators": Indicator}
            _field_encodings = {"probability": "decimal_string"}

        account: str
        """
        Account ID that this signal is associated with.
        """
        evaluated_at: str
        """
        Timestamp when the signal was evaluated.
        """
        fraudulent_merchant: Optional[FraudulentMerchant]
        """
        Fraudulent merchant signal data. Present when type is fraudulent_merchant.
        """
        id: str
        """
        Unique identifier for this account signal.
        """
        type: Literal["fraudulent_merchant"]
        """
        The type of account signal. Currently only fraudulent_merchant is supported.
        """
        _inner_class_types = {"fraudulent_merchant": FraudulentMerchant}

    data: V2SignalsAccountSignalFraudulentMerchantReadyEventData
    """
    Data for the v2.signals.account_signal.fraudulent_merchant_ready event
    """

    @classmethod
    def _construct_from(
        cls,
        *,
        values: Dict[str, Any],
        last_response: Optional[StripeResponse] = None,
        requestor: "_APIRequestor",
        api_mode: ApiMode,
    ) -> "V2SignalsAccountSignalFraudulentMerchantReadyEvent":
        evt = super()._construct_from(
            values=values,
            last_response=last_response,
            requestor=requestor,
            api_mode=api_mode,
        )
        if hasattr(evt, "data"):
            evt.data = V2SignalsAccountSignalFraudulentMerchantReadyEvent.V2SignalsAccountSignalFraudulentMerchantReadyEventData._construct_from(
                values=evt.data,
                last_response=last_response,
                requestor=requestor,
                api_mode=api_mode,
            )
        return evt
